@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# $Id: mysql_rotate.pl,v 1.2 2004/11/21 22:24:00 anton Exp $
+# $Id: mysql_rotate.pl,v 1.3 2008-01-02 13:31:38 anton Exp $
 # 
 # This script removes old unnecessary data from netams tables.
 # 
@@ -22,13 +22,18 @@ $time = timelocal(0,0,0,1,$mon,$year);
 
 # Data required to do a script login, change this
 # login to netams
-$mysql_dbname="netams"; $sc_host="localhost"; $sc_port=20001; $sc_user="admin"; $sc_passwd="123";
+$sql_type="mysql";
+$sql_dbname="netams";
+$sql_host="localhost";
+$sql_port=20001;
+$sql_login="admin";
+$sql_password="123";
 
-$dbh = DBI->connect("DBI:mysql:database=$mysql_dbname;host=$mysql_host", $mysql_login, $mysql_password);
+$dbh = DBI->connect("DBI:$sql_type:database=$sql_dbname;host=$sql_host", $sql_login, $sql_password);
 	if ($dbh eq "") {
                 $p1.="Unable to connect to SQL radius database!\n"; }
         else {
-                $p1.="Connected to database $mysql_dbname \n";
+                $p1.="Connected to database $sql_dbname \n";
         }
 
 #clear raw 
