@@ -1,27 +1,34 @@
 /*************************************************************************
 ***	Authentication, authorization, accounting + firewalling package
-***	(c) 1998-2002 Anton Vinokurov <anton@netams.com>
-***	(c) 2002-2005 NeTAMS Development Team
-***	All rights reserved. See 'Copying' file included in distribution
+***	Copyright 1998-2002 Anton Vinokurov <anton@netams.com>
+***	Copyright 2002-2008 NeTAMS Development Team
+***	This code is GPL v3
 ***	For latest version and more info, visit this project web page
 ***	located at http://www.netams.com
 ***
 *************************************************************************/
-/* $Id: netflow.c,v 1.32 2005/01/17 13:13:21 jura Exp $ */
+/* $Id: netflow.c,v 1.43 2009-08-01 09:23:55 anton Exp $ */
 
 #include "config.h"
-#include "lib.h"
+extern "C" {
+	#include "lib.h"
+}
 
 #define NETFLOW_CLASS
 #include "netflow.h"
 
-void Debug(char *msg,...);
+void Debug(const char *msg,...);
+
+#ifdef DEBUG
+static char buf1[32], buf2[32];
+#endif
+
 
 #define aMalloc(x) calloc(1,x)
 #define aFree(x)   free(x)
 
 //////////////////////////////////////////////////////////////////////////
-NetFlow::NetFlow(char *dst_host, unsigned short dst_port) {
+NetFlow::NetFlow(const char *dst_host, u_short dst_port) {
 	bzero(&udp_dest, sizeof(udp_dest));
     	udp_dest.sin_family=AF_INET;
     	udp_dest.sin_port=htons(dst_port);
@@ -32,7 +39,7 @@ NetFlow::NetFlow(char *dst_host, unsigned short dst_port) {
         	fprintf(stderr, "failed to create UDP socket: %s\n", strerror(errno));
         	exit(5);
     	}
-	
+
 	int status=bigsockbuf(udp_socket_fd, SO_SNDBUF, FT_SO_RCV_BUFSIZE);
 	if (status<0) {
 		fprintf(stderr, "failed to setsockopt send buffer: %s", strerror(errno));
@@ -40,7 +47,7 @@ NetFlow::NetFlow(char *dst_host, unsigned short dst_port) {
 	} else {
 		Debug("send bufer set to %u\n", status);
 	}
-	
+
 	t_active=60;
 	t_inactive=60;
 	t_expired=7;
@@ -51,16 +58,17 @@ NetFlow::NetFlow(char *dst_host, unsigned short dst_port) {
 	t_now=t=t_msg=time(NULL);
 	table=NULL;
 	ready=NULL;
-	last_id=0; 
-	expired_id=0;
-	sent_flows=0;
-	sent_packets=0;
-	table=(entry**)aMalloc(IPV4_HASH_SIZE*sizeof(entry*));
+	active=NULL;
+
+	last_id = expired_id = sent_flows = sent_packets = 0;
+
+	table=(entry_hash*)aMalloc(IPV4_HASH_SIZE*sizeof(entry_hash));
+
 	message=(IPStat5Msg*)aMalloc(sizeof(IPStat5Msg));
         message->header.version=htons(FLOW_VERSION_5);
         message->header.count=0;
 	message->header.unix_secs=t_now; //see PR 65
-	message->header.SysUptime=0; //see PR 65
+	message->header.sysUpTime=0; //see PR 65
         message->header.engine_id=0;
         message->header.engine_type=htons(31);
         message->header.flow_sequence=htonl(sent_flows);
@@ -68,11 +76,12 @@ NetFlow::NetFlow(char *dst_host, unsigned short dst_port) {
 //////////////////////////////////////////////////////////////////////////
 NetFlow::~NetFlow() {
 	FlushAll();
+
 	entry *ptr=ready;
 	while(ready) {
 		ptr=ready;
 		ready=ready->next;
-	aFree(ptr);
+		aFree(ptr);
 	}
 	aFree(table);
 	aFree(message);
@@ -89,18 +98,19 @@ void NetFlow::SetTimeouts(unsigned active,unsigned inactive,unsigned expired, un
 void NetFlow::CheckExpire(entry *e){
     	if ((unsigned)e->last - (unsigned)e->start > t_active) e->expired=1;
     	else if (t_now - (unsigned)e->last > t_inactive) e->expired=1;
-#ifdef DEBUG 
-	if(e->expired) Debug("%u expired\n", e->id);
+#ifdef DEBUG
+	if(e->expired) Debug("%p expired\n", e);
 #endif
 }
 //////////////////////////////////////////////////////////////////////////
 void NetFlow::Processpacket(struct ip *ip) {
-    	unsigned hash;
+    	u_short hash;
 	u_short sp=0, dp=0;
 	in_addr_t src,dst;
 	u_char proto;
 	u_char tos;
 	entry *e;
+	entry_hash *h;
 	u_short chunk=0;
 
 	dst = ip->ip_dst.s_addr;
@@ -112,7 +122,7 @@ void NetFlow::Processpacket(struct ip *ip) {
 		//packet fragmented. check obtained from /usr/src/sys/netinet/ip_fastfwd.c
 		sp=dp=0;
 		hash=IPV4_ADDR_HASH(src, dst);
-        } else 
+        } else
 	if (ip->ip_p==IPPROTO_TCP || ip->ip_p==IPPROTO_UDP) {
 		struct tcphdr *th;
 		th=(struct tcphdr *)((unsigned char *)ip + ip->ip_hl*4);
@@ -122,8 +132,10 @@ void NetFlow::Processpacket(struct ip *ip) {
 	} else {
 		hash=IPV4_ADDR_HASH(src, dst);
 	}
-	
-	for(e=table[hash]; e!=NULL; e=e->next){
+
+	h = &table[hash];
+
+	for(e=h->root; e!=NULL; e=e->next){
 		if (e->ip_dst==dst && e->ip_src==src &&
 		e->ip_proto==proto && e->ip_tos==tos &&
 		e->src_port==sp && e->dst_port==dp) {
@@ -135,7 +147,7 @@ void NetFlow::Processpacket(struct ip *ip) {
         	}
 		chunk++;
     	}
-	
+
 	if (e==NULL) {
 		if(chunk > MAX_CHUNK) FlushAll(); //protection against DoS
 		if(ready) { //we'll use ready empty entry
@@ -155,99 +167,105 @@ void NetFlow::Processpacket(struct ip *ip) {
         	e->ip_tos=tos;
         	e->start=e->last=t_now;
         	e->expired=0;
-        	e->id=last_id;
         	e->count=1;
         	last_id++;
-        	
-		e->next=table[hash];
-		table[hash]=e;
-		
-#ifdef DEBUG
-        	char buf1[32], buf2[32];
-        	strcpy(buf1, inet_ntoa(ip->ip_src));
-        	strcpy(buf2, inet_ntoa(ip->ip_dst));
 
-      		Debug("[%u(%u)]-%p %u s:%s:%u d:%s:%u %u\n", hash, e->id, e, proto, buf1, ntohs(sp), buf2, ntohs(dp), ntohs(ip->ip_len));
+		if(h->root == NULL) {
+			//organize active list
+			h->next_active = active;
+			active = h;
+		}
+
+		//plug new entry
+		e->next=h->root;
+		h->root=e;
+#ifdef DEBUG
+		inet_ntop(AF_INET, &(ip->ip_src), buf1, 32);
+		inet_ntop(AF_INET, &(ip->ip_dst), buf2, 32);
+		Debug("[%u]-%p %u s:%s:%u d:%s:%u %u\n", hash, e, proto, buf1, ntohs(sp), buf2, ntohs(dp), ntohs(ip->ip_len));
 #endif
 	}
 }
 //////////////////////////////////////////////////////////////////////////
 void NetFlow::Expiresearch(){
-    	unsigned i;
-    	entry *e, *prev;
-	entry *tmp;
-	
+
 	t_now=time(NULL);
 	if((unsigned)(t_now-t)<t_expired) return;
 	t=t_now;
-		
-    	Debug("ExpireSearch t:[%u] a:[%u] at %lu: \n", last_id,last_id-expired_id, t_now);
-    	
-	for (i=0; i<IPV4_HASH_SIZE; i++) {
-        	prev=NULL;
-		e=table[i];
-		while(e) {	
-			tmp=e->next;
-            		CheckExpire(e);
-            		if (e->expired) {
-				// with high probability used packet might be used again thou ...
-				if(e->count) {
-					ToSend(e);
-					e->count=0;
-					e->ip_len=0;	
-					e->expired=0;
-					e->start=e->last=t_now;
-//					e->id=last_id;
-//                			last_id++;
-					prev=e;
-                		} else {
-					expired_id++;
-					if (e==table[i])
-                                        	table[i]=(entry *)e->next;
-                                	else {
-                                        	prev->next=e->next;
-                                	}
-					//this means we have flow that was unaccessible one round
-					//move it ready 
-					e->next=ready;
-					ready=e;
-					e_used--;
-				}
-            		} else {
-            			prev=e;
+
+	entry_hash *prev_h=NULL;
+	entry_hash *next_h;
+
+    	Debug("ExpireSearch t:[%u] a:[%u] at %u: \n", last_id,last_id-expired_id, t_now);
+
+	for(entry_hash *h = active; h!=NULL; h = next_h) {
+	    next_h = h->next_active;
+
+	    entry *prev_e=NULL;
+	    entry *next_e;
+	    for(entry *e = h->root; e!=NULL; e = next_e) {
+		next_e = e->next;
+
+       		CheckExpire(e);
+           	if (e->expired) {
+		// with high probability used packet might be used again thou ...
+			if(e->count) {
+				ToSend(e);
+				e->count=0;
+				e->ip_len=0;
+				e->expired=0;
+				e->start=e->last=t_now;
+               		} else {
+				expired_id++;
+
+				//remove from table
+				if(e == h->root) h->root = e->next;
+				else prev_e->next = e->next;
+
+				//this means we have flow that was unaccessible one round
+				//move it ready
+				e->next=ready;
+				ready=e;
+				e_used--;
+				continue;
 			}
-			e=tmp;
-        	}
+       		}
+		prev_e = e;
+	    }
+	    //remove non acive brances from list
+	    if(h->root == NULL) {
+	    	if(h == active) active = h->next_active;
+		else prev_h->next_active = h->next_active;
+	    } else
+	    	prev_h = h;
     	}
+
 	if(unsigned(t-t_msg) > t_msg_expired) DoSend();
 }
 //////////////////////////////////////////////////////////////////////////
 void NetFlow::FlushAll() {
-    	unsigned i;
-    	entry *e;
-	entry *tmp;
+    	entry *next_e;
 
     	Debug("FlushAll::begin\n");
-        Debug("FlushAll t:[%u] a:[%u] at %lu: ", last_id, last_id-expired_id, t_now);
+        Debug("FlushAll t:[%u] a:[%u] at %u: ", last_id, last_id-expired_id, t_now);
 
-    	for (i=0; i<IPV4_HASH_SIZE; i++) {
-		e=table[i];
-		while(e) {
-                	Debug("%u{%u} ", e->id, i);
-            		expired_id++;
-			tmp=e->next;
-            		if(e->count) ToSend(e);
+	for (entry_hash *h = active; h!=NULL; h = h->next_active) {
+		for(entry *e = h->root; e!=NULL; e = next_e) {
+			next_e = e->next;
+
+			expired_id++;
+
+			if(e->count) ToSend(e);
 			e->next=ready;
 			ready=e;
 			e_used--;
-            		e=tmp;
 		}
-		table[i]=NULL;
+		h->root=NULL;
     	}
-	
+	active = NULL;
+
 	// it's time to check if there is stalled unsent data in message
 	DoSend();
-
     	Debug("FlushAll::end\n");
 }
 //////////////////////////////////////////////////////////////////////////
@@ -263,7 +281,7 @@ void NetFlow::ToSend(entry *e) {
 
         record->tos=e->ip_tos;
         record->prot=e->ip_proto;
-        record->First=htonl((e->start-message->header.unix_secs)*1000);   //trick here, see PR 65 
+        record->First=htonl((e->start-message->header.unix_secs)*1000);   //trick here, see PR 65
         record->Last=htonl((e->last-message->header.unix_secs)*1000);   //trick here, see PR 65
 
         record->dst_as=0;
@@ -278,7 +296,7 @@ void NetFlow::ToSend(entry *e) {
 
         message->header.count++;
         Debug("record %u in packet %llu prepared, %u\n", message->header.count-1, sent_packets+1, e->ip_len);
-        if (message->header.count==V5FLOWS_PER_PAK) DoSend();
+        if (message->header.count==V5_MAXFLOWS) DoSend();
 }
 //////////////////////////////////////////////////////////////////////////
 void NetFlow::DoSend() {
@@ -287,7 +305,7 @@ void NetFlow::DoSend() {
     	sent_packets++;
     	sent_flows=sent_flows+message->header.count;
         Debug("sending NetFlow v5 packet with %u flows\n", message->header.count);
-	
+
 	unsigned count=message->header.count;
     	message->header.count=htons(message->header.count);
 	message->header.unix_secs=htonl(message->header.unix_secs);
@@ -309,7 +327,7 @@ void NetFlow::Status() {
 //////////////////////////////////////////////////////////////////////////
 u_char debug=0, quiet=0;
 
-void Debug(char *msg,...){
+void Debug(const char *msg,...){
 #ifdef DEBUG
 	va_list args;
 	char *params;
